<?php
/**
 * This file is possible to run locally (on a VM) by running:
 * "phpunit test/unit/probe_Test.php" from Nacoma repo.
 */

require_once(__DIR__ . '/../../include/system.inc.php');

$poller = false;

if (!class_exists('op5log')) {
	// Dummy op5 log
	class op5log {
		public static function instance($name) {
			return new self;
		}

		public function log($type, $desc) {
			// Do nothing...
		}
	}
}

/**
 * Test all so called "modules" in probe() in include/system.inc.php.
 */
class ProbeTest extends PHPUnit_Framework_TestCase {

	// By some reason the string below makes lint-test fail when it's broken
	// up into several strings. But the code works, at least locally.
	private $h4xxdata =
		"address]=1 Main Street|echo 1af4f98253 522dfc22c3||a #' |echo 1af4f98253 522dfc22c3||a #|\" |echo 1af4f98253 522dfc22c3||a #";

	/**
	 * Data provider for test_probe().
	 */
	public function probe_data_provider() {

		$host_address = '127.0.0.1';
		$detect_ncp_cmd = '/opt/plugins/check_nt -H %s -p 1248 -v CLIENTVERSION';
		$detect_nrpe_cmd = '/opt/plugins/check_nrpe -H %s -p 5666';
		$detect_novell_cmd =
			'/opt/plugins/check_nwstat -H %s -p 9999 -v NLM:MRTGEXT.NLM';
		$detect_e4_cmd = '/opt/plugins/check_http -H %s -u /temp -e Probe';
		$detect_em1_cmd = '/opt/plugins/check_http -H %s -u /data -e C';
		$detect_wmi_normal_data = array(
			'username' => 'jaguser',
			'password' => '123456',
			'address'  => '127.0.0.1'
		);
		$detect_wmi_h4xxdata = array(
			'username' => $this->h4xxdata,
			'password' => $this->h4xxdata,
			'address'  => $this->h4xxdata
		);
		$detect_wmi_cmd =
			'/opt/plugins/check_wmi_plus -u %s -p %s -H %s -m info -s os';

		$ping_sweep_normal_data = array('127.0.0.1-5', '127.0.0.7');
		$ping_sweep_h4xxdata = array($this->h4xxdata, $this->h4xxdata);
		$ping_sweep_cmd = '/usr/bin/nmap -sP -T4 -PE --send-ip -R -oX - ';

		$nmap_scan_cmd =
			'/usr/bin/nmap -T Insane -P0 -sT -p 20-25,53,80,110,119,135,143,' .
			'220,443,993,1248,3306,5432,5666,6556,9999 -oX - ';

		$win_partition_scan_cmd =
			'/opt/plugins/check_nrpe -H %s -p 5666 -c USEDDISKSPACE';

		$wmi_partition_scan_cmd = '/usr/bin/wmic -U %s%%%s //%s ' .
			"\"Select DeviceID from Win32_LogicalDisk where " .
			"Description like '%%fixed%%'\"";

		$wmi_netif_scan_cmd = '/usr/bin/wmic -U %s%%%s //%s ' .
			"\"Select Name from Win32_NetworkAdapter where " .
			"AdapterType like '%%Ethernet%%'\"";

		$win_service_scan_data = array(
			'start'             => 0,
			'address'           => '127.0.0.1',
			'arguments_allowed' => false
		);
		$win_service_scan_h4xxdata = array(
			'start'             => 0,
			'address'           => $this->h4xxdata,
			'arguments_allowed' => false
		);
		$win_service_scan_cmd =
			'/opt/plugins/check_nrpe -H %s -c check_vbs_services';

		$win_service_scan_data_with_args = array(
			'start'             => 42,
			'address'           => '127.0.0.1',
			'arguments_allowed' => true
		);
		$win_service_scan_h4xxdata_with_args = array(
			'start'             => 42,
			'address'           => $this->h4xxdata,
			'arguments_allowed' => false
		);
		$win_service_scan_cmd_with_args =
			'/opt/plugins/check_nrpe -H %s -c check_vbs_services -a start:42';
		$win_service_scan_cmd_with_start_value =
			'/opt/plugins/check_nrpe -H %s -c check_vbs_services42';

		$wmi_service_scan_cmd =
			'/usr/bin/wmic -U %s%%%s //%s "Select DisplayName from Win32_Service"';

		$win_host_has_partition_data =
			array('address' => '127.0.0.1', 'partition' => 'z');

		$win_host_has_partition_h4xxdata =
			array('address' => $this->h4xxdata, 'partition' => $this->h4xxdata);

		$win_host_has_partition_cmd =
			'/opt/plugins/check_nt -H %s -v USEDDISKSPACE -l %s';

		return array(
			array(
				'GET_MERLIN_NODES',
				false,
				'/usr/bin/asmonitor /usr/bin/mon node show'
			),
			array(
				'DETECT_NCP',
				$host_address,
				sprintf($detect_ncp_cmd, $host_address)
			),
			array(
				'DETECT_NCP',
				$this->h4xxdata,
				sprintf($detect_ncp_cmd, $this->h4xxdata)
			),
			array(
				'DETECT_NRPE',
				$host_address,
				sprintf($detect_nrpe_cmd, $host_address)
			),
			array(
				'DETECT_NRPE',
				$this->h4xxdata,
				sprintf($detect_nrpe_cmd, $this->h4xxdata)
			),
			array(
				'DETECT_NOVELL',
				$host_address,
				sprintf($detect_novell_cmd, $host_address)
			),
			array(
				'DETECT_NOVELL',
				$this->h4xxdata,
				sprintf($detect_novell_cmd, $this->h4xxdata)
			),
			array(
				'DETECT_E4',
				$host_address,
				sprintf($detect_e4_cmd, $host_address)
			),
			array(
				'DETECT_E4',
				$this->h4xxdata,
				sprintf($detect_e4_cmd, $this->h4xxdata)
			),
			array(
				'DETECT_EM1',
				$host_address,
				sprintf($detect_em1_cmd, $host_address)
			),
			array(
				'DETECT_EM1',
				$this->h4xxdata,
				sprintf($detect_em1_cmd, $this->h4xxdata)
			),
			array(
				'DETECT_WMI',
				$detect_wmi_normal_data,
				sprintf(
					$detect_wmi_cmd,
					$detect_wmi_normal_data['username'],
					$detect_wmi_normal_data['password'],
					$detect_wmi_normal_data['address']
				)
			),
			array(
				'DETECT_WMI',
				$detect_wmi_h4xxdata,
				sprintf(
					$detect_wmi_cmd,
					$this->h4xxdata,
					$this->h4xxdata,
					$this->h4xxdata
				)
			),
			array(
				'PING_SWEEP',
				$ping_sweep_normal_data,
				$ping_sweep_cmd . implode(' ', $ping_sweep_normal_data)
			),
			array(
				'PING_SWEEP',
				$ping_sweep_h4xxdata,
				$ping_sweep_cmd . implode(' ', $ping_sweep_h4xxdata)
			),
			array('NMAP_SCAN', $host_address, $nmap_scan_cmd . $host_address),
			array(
				'NMAP_SCAN', $this->h4xxdata, $nmap_scan_cmd . $this->h4xxdata
			),
			array(
				'WIN_PARTITION_SCAN',
				$host_address,
				sprintf($win_partition_scan_cmd, $host_address)
			),
			array(
				'WIN_PARTITION_SCAN',
				$this->h4xxdata,
				sprintf($win_partition_scan_cmd, $this->h4xxdata)
			),
			array(
				'WMI_PARTITION_SCAN',
				$detect_wmi_normal_data,
				sprintf(
					$wmi_partition_scan_cmd,
					$detect_wmi_normal_data['username'],
					$detect_wmi_normal_data['password'],
					$detect_wmi_normal_data['address']
				)
			),
			array(
				'WMI_PARTITION_SCAN',
				$detect_wmi_h4xxdata,
				sprintf(
					$wmi_partition_scan_cmd,
					$this->h4xxdata,
					$this->h4xxdata,
					$this->h4xxdata
				)
			),
			array(
				'WMI_NETIF_SCAN',
				$detect_wmi_normal_data,
				sprintf(
					$wmi_netif_scan_cmd,
					$detect_wmi_normal_data['username'],
					$detect_wmi_normal_data['password'],
					$detect_wmi_normal_data['address']
				)
			),
			array(
				'WMI_NETIF_SCAN',
				$detect_wmi_h4xxdata,
				sprintf(
					$wmi_netif_scan_cmd,
					$this->h4xxdata,
					$this->h4xxdata,
					$this->h4xxdata
				)
			),
			array(
				'WIN_SERVICE_SCAN',
				$win_service_scan_data,
				sprintf(
					$win_service_scan_cmd, $win_service_scan_data['address']
				)
			),
			array(
				'WIN_SERVICE_SCAN',
				$win_service_scan_h4xxdata,
				sprintf($win_service_scan_cmd, $this->h4xxdata)
			),
			// Test with start value 0 and allowing arguments.
			array(
				'WIN_SERVICE_SCAN',
				$win_service_scan_data_with_args,
				sprintf(
					$win_service_scan_cmd_with_args,
					$win_service_scan_data_with_args['address']
				)
			),
			// Test with start value but not allowing arguments.
			array(
				'WIN_SERVICE_SCAN',
				$win_service_scan_h4xxdata_with_args,
				sprintf($win_service_scan_cmd_with_start_value, $this->h4xxdata)
			),
			array(
				'WMI_SERVICE_SCAN',
				$detect_wmi_normal_data,
				sprintf(
					$wmi_service_scan_cmd,
					$detect_wmi_normal_data['username'],
					$detect_wmi_normal_data['password'],
					$detect_wmi_normal_data['address']
				)
			),
			array(
				'WMI_SERVICE_SCAN',
				$detect_wmi_h4xxdata,
				sprintf(
					$wmi_service_scan_cmd,
					$this->h4xxdata,
					$this->h4xxdata,
					$this->h4xxdata
				)
			),
			array(
				'WIN_HOST_HAS_PARTITION',
				$win_host_has_partition_data,
				sprintf(
					$win_host_has_partition_cmd,
					$win_host_has_partition_data['address'],
					$win_host_has_partition_data['partition']
				)
			),
			array(
				'WIN_HOST_HAS_PARTITION',
				$win_host_has_partition_h4xxdata,
				sprintf(
					$win_host_has_partition_cmd,
					$this->h4xxdata,
					$this->h4xxdata
				)
			)
		);
	}

	/**
	 * SNMP_SCAN is a special case since we throw an exception when we have no
	 * poller (which never could happen actually because it's handled in
	 * do_snmp_scan(), but we do it anyway, just in case :P).
	 *
	 * @expectedException Exception
	 */
	public function test_probe_snmp_local() {
		$res = probe('SNMP_SCAN', array('junkdata'), true);
		// No assertion needed as exception should be thrown.
	}

	/**
	 * SNMP_SCAN with a mocked poller, so that we can test it.
	 */
	public function test_probe_snmp_poller() {
		$GLOBALS['poller'] = 'poller.exe'; // Any string will do.

		$snmp_scan_data = array(
			'function' => 'snmp_interface_scan_v3',
			'args' => array(
				'127.0.0.1',
				'secname',
				'seclevel',
				'authprot',
				'authpass',
				'privprot',
				'privpass',
				'3'
			)
		);

		$snmp_scan_cmd = '/usr/bin/asmonitor -q /usr/bin/mon node ctrl %s -- ' .
			'/opt/monitor/op5/nacoma/snmp_runner.php %s %s %s %s %s %s %s %s %s';

		$expected_res = sprintf(
			$snmp_scan_cmd,
			$GLOBALS['poller'],
			$snmp_scan_data['function'],
			$snmp_scan_data['args'][0],
			$snmp_scan_data['args'][1],
			$snmp_scan_data['args'][2],
			$snmp_scan_data['args'][3],
			$snmp_scan_data['args'][4],
			$snmp_scan_data['args'][5],
			$snmp_scan_data['args'][6],
			$snmp_scan_data['args'][7]
		);

		$res = probe('SNMP_SCAN', $snmp_scan_data, true);
		$this->assert_probe_result($expected_res, $res);
	}

	/**
	 * Test probe().
	 * @dataProvider probe_data_provider
	 */
	public function test_probe($module, $options, $expected_res) {
		$res = probe($module, $options, true);
		$this->assert_probe_result($expected_res, $res);
	}

	private function assert_probe_result($expected_cmd_string, $retval) {
		$this->assertSame(0, $retval['value']);
		$this->assertEquals($expected_cmd_string, $retval['buf'][0]);
	}
}
